==> sife-0.0.0.tgz: package/lib/componentBuilder.js <==
var fs = require('fs'),
	_ = require('underscore');


module.exports = {

	init: function(conf) {
		this.conf = conf;
	},

	/**
	 * Creates all needed tasks for all components
	 * @param  {gruntHelper} gruntHelper
	 * @param  {string} taskName
	 * @return {void}
	 */
	createTasks: function(gruntHelper, taskName) {

		var components = this.getComponentsFromFiles(this.conf.sources.components);

		_.each(components, function(component) {

			var files = this.getComponentDependencyFiles(component);
			gruntHelper.concat.add(taskName, this.conf.output.css.components + '/' + component.id + '.css', files);

		}, this);

	},


	getComponentsFromFiles: function(path) {

		var namespaces = fs.readdirSync(path),
			components = [];

		_.each(namespaces, function(namespace) {

			if (fs.statSync(path + '/' + namespace).isDirectory()) {
				this.addComponentsFromNamespace(namespace, path, components);
			}

		}, this);

		return components;

	},


	addComponentsFromNamespace: function(namespace, path, components) {

		var comps = fs.readdirSync(path + '/' + namespace),
			comp,
			namespaceOptions = this.conf.sources.componentsOptions[namespace] || {};

		// ignore namespsaces that are 'cssOnly' (typically utilities or variables)
		if (namespaceOptions.cssOnly) {
			return;
		}

		_.each(comps, function(componentName) {
			if (namespaceOptions.atomic) {

				var atomicEntity = componentName;
				var atomicEntities = fs.readdirSync(path + '/' + namespace + '/' + atomicEntity);

				_.each(atomicEntities, function(componentName) {

					comp = {
						name: componentName,
						namespace: namespace,
						path: path + '/' + namespace + '/' + atomicEntity + '/' + componentName,
						dependencies: this.findComponentDependencies(namespace + '/' + atomicEntity + '/' + componentName),
						id: namespace + '/' + atomicEntity + '/' + componentName
					};
					components.push(comp);

				}, this);

			}

			else {

				comp = {
					name: componentName,
					namespace: namespace,
					path: path + '/' + namespace + '/' + componentName,
					dependencies: this.findComponentDependencies(namespace + '/' + componentName),
					id: namespace + '/' + componentName
				};
				components.push(comp);

			}


		}, this);


	},


	/**
	 * Returns an object containing following keys:
	 * - explicit: array of components described in config.json
	 * - variations: array of variations as found in css/ folder of component
	 * - layout: componentString of the componenet's layout
	 * @param  {[type]} componentString [description]
	 * @return {[type]}                 [description]
	 */
	findComponentDependencies: function(componentString) {

		var config = this.getComponentConfig(componentString);
		var deps = {};

		if (config.dependencies) {
			deps.explicit = config.dependencies;
		}

		var variations = this.getComponentVariations(componentString);

		if (variations) {
			deps.variations = variations;
		}

		if (config.layout) {
			deps.layout = config.layout;
		}

		return deps;

	},


	/**
	 * Returns the given component's config
	 * (from config.json) or {} if no config.json
	 * @param  {componentString} componentString
	 * @return {object}
	 */
	getComponentConfig: function(componentString) {

		var fileContent;

		try {
			fileContent = fs.readFileSync(this.conf.sources.components + '/' + componentString + '/config.json');
		}

		catch (err) {
			fileContent = '{}';
		}

		return JSON.parse(fileContent);

	},


	/**
	 * Returns an array of all variations for a given componentString
	 * @param  {string} componentString
	 * @return {array} or null if none
	 */
	getComponentVariations: function(componentString) {

		var variations;

		try {
			variations = fs.readdirSync(this.conf.sources.components + '/' + componentString + '/css');
		}

		catch (err) {
			variations = null;
		}

		if (variations) {

			variations = _.map(variations, function(f) {
				return f.replace('.css', '');
			});
			variations = _.filter(variations, function(f) {
				return f.match(/--/);
			});

			if (variations.length === 0) {
				variations = null;
			}

		}

		return variations;

	},

	/**
	 * Returns an array of CSS files the given component
	 * depends on
	 * @param  {object} component Component object
	 * @return {array}
	 */
	getComponentDependencyFiles: function(component) {

		// [options.prependFiles]
		// the component css
		// component's dependencies (--variations)
		// excplicit dependencies (in config.json)
		// layout dependencies
		var files = [];

		// for all components, prepend files
		if (this.conf.build.componentsOptions.prependFiles) {
			files = files.concat(this.conf.build.componentsOptions.prependFiles);
		}

		// the component's own CSS
		files.push(component.path + '/css/' + component.name + '.css');

		// all variations CSS
		if (component.dependencies.variations) {
			files = files.concat(_.map(component.dependencies.variations, function(variation) {
				return component.path + '/css/' + variation + '.css';
			}));
		}

		// the layout CSS
		if (component.dependencies.layout) {

			var layoutName = component.dependencies.layout.split('/').pop();
			files.push(this.conf.output.css.components + '/' + component.dependencies.layout + '/css/' + layoutName + '.css');

		}

		return files;

	}
};

==> sife-0.0.2.tgz: package/lib/componentBuilder.js <==
var fs = require('fs'),
	_ = require('underscore');


module.exports = {

	init: function(conf) {
		this.conf = conf;
	},

	/**
	 * Creates all needed tasks for all components
	 * @param  {gruntHelper} gruntHelper
	 * @param  {string} taskName
	 * @return {void}
	 */
	createTasks: function(gruntHelper, taskName) {

		var components = this.getComponentsFromFiles(this.conf.sources.components);

		_.each(components, function(component) {

			var files = this.getComponentDependencyFiles(component);
			gruntHelper.concat.add(taskName, this.conf.output.css.components + '/' + component.id + '.css', files);

		}, this);

	},


	getComponentsFromFiles: function(path) {

		var namespaces = fs.readdirSync(path),
			components = [];

		_.each(namespaces, function(namespace) {

			if (fs.statSync(path + '/' + namespace).isDirectory()) {
				this.addComponentsFromNamespace(namespace, path, components);
			}

		}, this);

		return components;

	},


	addComponentsFromNamespace: function(namespace, path, components) {

		var comps = fs.readdirSync(path + '/' + namespace),
			comp,
			namespaceOptions = this.conf.sources.componentsOptions[namespace] || {};

		// ignore namespsaces that are 'cssOnly' (typically utilities or variables)
		if (namespaceOptions.cssOnly) {
			return;
		}

		_.each(comps, function(componentName) {
			if (namespaceOptions.atomic) {

				var atomicEntity = componentName;
				var atomicEntities = fs.readdirSync(path + '/' + namespace + '/' + atomicEntity);

				_.each(atomicEntities, function(componentName) {

					comp = {
						name: componentName,
						namespace: namespace,
						path: path + '/' + namespace + '/' + atomicEntity + '/' + componentName,
						dependencies: this.findComponentDependencies(namespace + '/' + atomicEntity + '/' + componentName),
						id: namespace + '/' + atomicEntity + '/' + componentName
					};
					components.push(comp);

				}, this);

			}

			else {

				comp = {
					name: componentName,
					namespace: namespace,
					path: path + '/' + namespace + '/' + componentName,
					dependencies: this.findComponentDependencies(namespace + '/' + componentName),
					id: namespace + '/' + componentName
				};
				components.push(comp);

			}


		}, this);


	},


	/**
	 * Returns an object containing following keys:
	 * - explicit: array of components described in config.json
	 * - variations: array of variations as found in css/ folder of component
	 * - layout: componentString of the componenet's layout
	 * @param  {[type]} componentString [description]
	 * @return {[type]}                 [description]
	 */
	findComponentDependencies: function(componentString) {

		var config = this.getComponentConfig(componentString);
		var deps = {};

		if (config.dependencies) {
			deps.explicit = config.dependencies;
		}

		var variations = this.getComponentVariations(componentString);

		if (variations) {
			deps.variations = variations;
		}

		if (config.layout) {
			deps.layout = config.layout;
		}

		return deps;

	},


	/**
	 * Returns the given component's config
	 * (from config.json) or {} if no config.json
	 * @param  {componentString} componentString
	 * @return {object}
	 */
	getComponentConfig: function(componentString) {

		var fileContent;

		try {
			fileContent = fs.readFileSync(this.conf.sources.components + '/' + componentString + '/config.json');
		}

		catch (err) {
			fileContent = '{}';
		}

		return JSON.parse(fileContent);

	},


	/**
	 * Returns an array of all variations for a given componentString
	 * @param  {string} componentString
	 * @return {array} or null if none
	 */
	getComponentVariations: function(componentString) {

		var variations;

		try {
			variations = fs.readdirSync(this.conf.sources.components + '/' + componentString + '/css');
		}

		catch (err) {
			variations = null;
		}

		if (variations) {

			variations = _.map(variations, function(f) {
				return f.replace('.css', '');
			});
			variations = _.filter(variations, function(f) {
				return f.match(/--/);
			});

			if (variations.length === 0) {
				variations = null;
			}

		}

		return variations;

	},

	/**
	 * Returns an array of CSS files the given component
	 * depends on
	 * @param  {object} component Component object
	 * @return {array}
	 */
	getComponentDependencyFiles: function(component) {

		// [options.prependFiles]
		// the component css
		// component's dependencies (--variations)
		// excplicit dependencies (in config.json)
		// layout dependencies
		var files = [];

		// for all components, prepend files
		if (this.conf.build.componentsOptions && this.conf.build.componentsOptions.prependFiles) {
			files = files.concat(this.conf.build.componentsOptions.prependFiles);
		}

		// the component's own CSS
		files.push(component.path + '/css/' + component.name + '.css');

		// all variations CSS
		if (component.dependencies.variations) {
			files = files.concat(_.map(component.dependencies.variations, function(variation) {
				return component.path + '/css/' + variation + '.css';
			}));
		}

		// the layout CSS
		if (component.dependencies.layout) {

			var layoutName = component.dependencies.layout.split('/').pop();
			files.push(this.conf.output.css.components + '/' + component.dependencies.layout + '/css/' + layoutName + '.css');

		}

		return files;

	}
};
